
From: Yoav Zach <yoav_zach@yahoo.com>

In ia64 kernel, the O_LARGEFILE flag is forced when opening a file.  This
is problematic for execution of 32 bit processes, which are not largefile
aware, either by SW emulation or by HW execution.

For such processes, the problem is two-fold:

1) When trying to open a file that is larger than 4G
   the operation should fail, but it's not
2) Writing to offset larger than 4G should fail, but
   it's not

The proposed patch takes advantage of the way 32 bit processes are
identified in ia64 systems.  Such processes have PER_LINUX32 for their
personality.  With the patch, the ia64 kernel will not enforce the
O_LARGEFILE flag if the current process has PER_LINUX32 set.  The behavior
for all other architectures remains unchanged.

Signed-off-by: Yoav Zach <yoav.zach@intel.com>
Acked-by: Tony Luck <tony.luck@intel.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 fs/open.c                |    7 ++++---
 include/asm-ia64/fcntl.h |    2 ++
 include/linux/fcntl.h    |    4 ++++
 3 files changed, 10 insertions(+), 3 deletions(-)

diff -puN fs/open.c~dont-force-o_largefile-for-32-bit-processes-on-ia64-2612-rc3 fs/open.c
--- 25/fs/open.c~dont-force-o_largefile-for-32-bit-processes-on-ia64-2612-rc3	2005-05-13 22:13:41.000000000 -0700
+++ 25-akpm/fs/open.c	2005-05-13 22:13:41.000000000 -0700
@@ -21,6 +21,7 @@
 #include <linux/vfs.h>
 #include <asm/uaccess.h>
 #include <linux/fs.h>
+#include <linux/personality.h>
 #include <linux/pagemap.h>
 #include <linux/syscalls.h>
 
@@ -935,9 +936,9 @@ asmlinkage long sys_open(const char __us
 	char * tmp;
 	int fd, error;
 
-#if BITS_PER_LONG != 32
-	flags |= O_LARGEFILE;
-#endif
+	if (force_o_largefile())
+		flags |= O_LARGEFILE;
+
 	tmp = getname(filename);
 	fd = PTR_ERR(tmp);
 	if (!IS_ERR(tmp)) {
diff -puN include/asm-ia64/fcntl.h~dont-force-o_largefile-for-32-bit-processes-on-ia64-2612-rc3 include/asm-ia64/fcntl.h
--- 25/include/asm-ia64/fcntl.h~dont-force-o_largefile-for-32-bit-processes-on-ia64-2612-rc3	2005-05-13 22:13:41.000000000 -0700
+++ 25-akpm/include/asm-ia64/fcntl.h	2005-05-13 22:13:41.000000000 -0700
@@ -81,4 +81,6 @@ struct flock {
 
 #define F_LINUX_SPECIFIC_BASE	1024
 
+#define force_o_largefile() ( ! (current->personality & PER_LINUX32) )
+
 #endif /* _ASM_IA64_FCNTL_H */
diff -puN include/linux/fcntl.h~dont-force-o_largefile-for-32-bit-processes-on-ia64-2612-rc3 include/linux/fcntl.h
--- 25/include/linux/fcntl.h~dont-force-o_largefile-for-32-bit-processes-on-ia64-2612-rc3	2005-05-13 22:13:41.000000000 -0700
+++ 25-akpm/include/linux/fcntl.h	2005-05-13 22:13:41.000000000 -0700
@@ -25,6 +25,10 @@
 
 #ifdef __KERNEL__
 
+#ifndef force_o_largefile
+#define force_o_largefile() (BITS_PER_LONG != 32)
+#endif
+
 #if BITS_PER_LONG == 32
 #define IS_GETLK32(cmd)		((cmd) == F_GETLK)
 #define IS_SETLK32(cmd)		((cmd) == F_SETLK)
_
